import { AbortSignal, HttpClient } from '@dynatrace-sdk/http-client';
import { ExecuteRequest } from '../models/execute-request';
import { QueryPollResponse } from '../models/query-poll-response';
import { QueryStartResponse } from '../models/query-start-response';
import { ApiClientOptions } from './api-client-options';
/**
 * Operations related to the Query execution.
 */
export declare class QueryExecutionClient {
    private httpClient;
    private shouldTransformDates;
    /**
     * @param {HttpClient} httpClientImplementation - You can provide custom http client as a parameter to constructor of a QueryExecutionClient. Custom http client should implement {HttpClient} interface.
     * @param {ApiClientOptions} [options] - Optional configuration for SDK Client.
     * @example
     * const queryExecutionClientNodeJs = new QueryExecutionClient(yourCustomImplementation);
     */
    constructor(httpClientImplementation: HttpClient, options?: ApiClientOptions);
    /**
     * Retrieves query status and final result from Grail.
     *
     * For information about the required permissions see the [Bucket and table permissions in Grail documentation](https://docs.dynatrace.com/docs/shortlink/assign-bucket-table-permissions).
     *
     * ### Overview:
     *
     * Polls the status of a Grail query. Returns the status of the query, including the result if the query finished.
     *
     * ### The response format:
     * The json response will contain the state of the query. If the query succeeded, the result will be included.
     *
     * The result has two main sections:
     * * The 'records' section contains the individual records, where each record consists of a set of fields and their corresponding values.
     * * The 'types' section describes the corresponding data types that a record field has.
     *
     * Every record has an implicit 'index' according to the position in the 'records' JSON array.
     * The types section has a list of 1..N possible type 'buckets'. Each such bucket has an 'indexRange' which indicates which
     * records will find their field types in which bucket. The index range has two values start & end and can be thought of as [startIndex, endIndex).
     *
     * A field part of a record with index 'i' will find its corresponding field type by first locating the bucket that satisfies:
     *
     * ```text
     * startIndex <= i <= endIndex
     * ```
     *
     * Once the bucket is found the 'mappings' object has an entry for all the fields that are part of that record with index 'i'.
     *
     *
     * Since enforcement of a particular schema is absent at ingestion time, it is possible to have records that share the same
     * field name but their values are of a different type. This phenomenon will hence forth be named as a **"collision"**.
     * When a collision does occur, we will create a new type 'bucket' that will have a different index range where
     * the new record field types will be placed.
     * It is guaranteed that every field of every record will have a corresponding type.
     * **Clients should always take the included types into account when consuming records!**
     *
     * @returns {Promise<QueryPollResponse>} The current status and results of the supplied query.
     *
     * @example <caption>Code example</caption>
     * import { queryExecutionClient } from "@dynatrace-sdk/client-query";
     *
     * const data = await queryExecutionClient.queryPoll({
     *   requestToken: "...",
     * });
     *
     * @throws {ErrorEnvelopeError} The supplied request is wrong. Either the query itself or other parameters are wrong. | An internal server error has occurred.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    queryPoll(config: {
        /** The request-token of the query. */ requestToken: string;
        /** The time a client is willing to wait for the query result. In case the query finishes within the specified timeout, the query result is returned. Otherwise, the query status is returned. */ requestTimeoutMilliseconds?: number;
        /** If set additional data will be available in the metadata section. */ enrich?: string;
        /** The dt-client-context header is an optional string parameter used for monitoring purposes. When included in a request, it helps retrieve information about the execution of the query. It shouldn't hold sensitive information. */ dtClientContext?: string;
        abortSignal?: AbortSignal;
    }): Promise<QueryPollResponse>;
    /**
     * Starts a Grail query.
     *
     * For information about the required permissions see the [Bucket and table permissions in Grail documentation](https://docs.dynatrace.com/docs/shortlink/assign-bucket-table-permissions).
     *
     * ### Overview:
     *
     * Executes a query and returns a list of records.
     *
     * For details about the query language see the [Dynatrace Query Language documentation](https://dt-url.net/bv03yk8).
     *
     * ### The response format:
     *
     * The json response will contain the state of the started query. If the query succeeded, the result will be included. Otherwise the response will contain a request token to reference the query in future polling requests.
     *
     * The result has two main sections:
     * * The 'records' section contains the individual records, where each record consists of a set of fields and their corresponding values.
     * * The 'types' section describes the corresponding data types that a record field has.
     *
     * Every record has an implicit 'index' according to the position in the 'records' JSON array.
     * The types section has a list of 1..N possible type 'buckets'. Each such bucket has an 'indexRange' which indicates which
     * records will find their field types in which bucket. The index range has two values start & end and can be thought of as [startIndex, endIndex).
     *
     * A field part of a record with index 'i' will find its corresponding field type by first locating the bucket that satisfies:
     *
     * ```text
     * startIndex <= i <= endIndex
     * ```
     *
     * Once the bucket is found the 'mappings' object has an entry for all the fields that are part of that record with index 'i'.
     *
     *
     * Since enforcement of a particular schema is absent at ingestion time, it is possible to have records that share the same
     * field name but their values are of a different type. This phenomenon will hence forth be named as a **"collision"**.
     * When a collision does occur, we will create a new type 'bucket' that will have a different index range where
     * the new record field types will be placed.
     * It is guaranteed that every field of every record will have a corresponding type.
     * **Clients should always take the included types into account when consuming records!**
     *
     * @returns {Promise<QueryStartResponse>} The final status and results of the supplied query if it finished within a supplied requestTimeoutMilliseconds. | The status of the query to start.
     *
     * @example <caption>Code example</caption>
     * import { queryExecutionClient } from "@dynatrace-sdk/client-query";
     *
     * const data = await queryExecutionClient.queryExecute({
     *   body: {
     *     query:
     *       'fetch events | filter event.type == "davis" AND davis.status != "CLOSED" | fields timestamp, davis.title, davis.underMaintenance, davis.status | sort timestamp | limit 10',
     *   },
     * });
     *
     * @throws {ErrorEnvelopeError} The supplied request is wrong. Either the query itself or other parameters are wrong. | Too many requests. | An internal server error has occurred. | Service is unavailable. | Client error. | Server error.
     * @throws {InsufficientPermission} Insufficient permissions.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    queryExecute(config: {
        body: ExecuteRequest;
        /** If set additional data will be available in the metadata section. */ enrich?: string;
        /** The dt-client-context header is an optional string parameter used for monitoring purposes. When included in a request, it helps retrieve information about the execution of the query. It shouldn't hold sensitive information. */ dtClientContext?: string;
        /** If set, query consumption limit will be enforced. */ enforceQueryConsumptionLimit?: boolean;
        abortSignal?: AbortSignal;
    }): Promise<QueryStartResponse>;
    /**
     * Cancels the query and returns the result if the query was already finished, otherwise discards it.
     *
     * For information about the required permissions see the [Bucket and table permissions in Grail documentation](https://docs.dynatrace.com/docs/shortlink/assign-bucket-table-permissions).
     *
     * ### Overview:
     *
     * Cancels a running Grail query and returns a list of records if the query already finished.
     *
     * ### The response format:
     *
     * If the query was already finished, a response body including the result will be returned. Otherwise the response will contain no body.
     *
     * The result has three main sections:
     * * the 'records' section contains the individual records, where each record consists of a set of fields and their corresponding values.
     * * the 'types' section describes the corresponding data types that a record field has.
     * * the 'metadata' section contains information about the query like 'analysisTimeframe', 'timezone' or 'locale'.
     *
     * Every record has an implicit 'index' according to the position in the 'records' JSON array.
     * The types section has a list of 1..N possible type 'buckets'. Each such bucket has an 'indexRange' which indicates which
     * records will find their field types in which bucket. The index range has two values start & end and can be thought of as [startIndex, endIndex).
     *
     * A field part of a record with index 'i' will find its corresponding field type by first locating the bucket that satisfies:
     *
     * ```text
     * startIndex <= i <= endIndex
     * ```
     *
     * Once the bucket is found the 'mappings' object has an entry for all the fields that are part of that record with index 'i'.
     *
     *
     * Since enforcement of a particular schema is absent at ingestion time, it is possible to have records that share the same
     * field name but their values are of a different type. This phenomenon will hence forth be named as a **"collision"**.
     * When a collision does occur, we will create a new type 'bucket' that will have a different index range where
     * the new record field types will be placed.
     * It is guaranteed that every field of every record will have a corresponding type.
     * **Clients should always take the included types into account when consuming records!**
     *
     * @returns {Promise<QueryPollResponse | void>} The query already finished.
     *
     * @example <caption>Code example</caption>
     * import { queryExecutionClient } from "@dynatrace-sdk/client-query";
     *
     * const data = await queryExecutionClient.queryCancel({
     *   requestToken: "...",
     * });
     *
     * @throws {ErrorEnvelopeError} The supplied request is wrong. Either the query itself or other parameters are wrong. | An internal server error has occurred.
     * @throws {InvalidResponseError}
     * @throws {ApiClientError}
     * @throws {ClientRequestError}
     */
    queryCancel(config: {
        /** The request-token of the query. */ requestToken: string;
        /** If set additional data will be available in the metadata section. */ enrich?: string;
        /** The dt-client-context header is an optional string parameter used for monitoring purposes. When included in a request, it helps retrieve information about the execution of the query. It shouldn't hold sensitive information. */ dtClientContext?: string;
        abortSignal?: AbortSignal;
    }): Promise<QueryPollResponse | void>;
}
export declare const queryExecutionClient: QueryExecutionClient;
