/** @ignore*/
export declare enum ErrorType {
    COMMON = "JS Error",
    HTTP = "Http Error"
}
/**
 * Basic error envelope, that enforces all error to have mandatory "error" property.
 * */
export interface ErrorResponseBody<T = CommonApiError> {
    /**
     * *error* object must have two mandatory fields (*code, message*) and may have two additional fields (*help, details*):
     * refer to the *CommonApiError* for more details.
     * */
    error: T;
}
/**
 * Common API error structure, established by Dynatrace API guidelines.
 * */
export interface CommonApiError<T extends Record<string, any> = Record<string, any>> {
    /** The error code should be set to the HTTP error code by default.<br>
     * The error code may be set to an API-specific error code which must be properly documented.*/
    code: number;
    /** The error message should be short and precise, it should not contain details.*/
    message: string;
    /**
     * An additional help field may be added which must be a URL to further information on how to deal with the error.<br>
     * This may be some detailed error documentation page or a link to the Dynatrace support system, etc.
     * */
    help?: string;
    /**
     * Additional details about the error may be added in a details field.
     * Refer to *CommonErrorDetails*
     * */
    details?: T;
}
/**
 * *CommonApiError* details object reference. Contains common API error information.
 *  This object is used to convey additional information about the error like e.g., which query parameter exactly violated a precondition. <br>
 *  Details may contain any fields to further describe the error
 * */
export interface CommonErrorDetails {
    /**
     * UUID string that represents a reference of the error into e.g., the log file of the service.
     * */
    errorRef?: string;
    /**
     * String containing a 32-character hex integer value that is used for tracing.
     * */
    traceId?: string;
    /**
     * String value representing a more detailed error information than the http response code alone.
     * Must be a single word in CamelCase, and all possible values must be documented.
     * */
    errorCode?: string;
    /**
     * An array of *ConstraintViolation* object, refer to ConstraintViolation object reference for more details.
     * */
    constraintViolations?: ConstraintViolation[];
    /**
     * Must be an array of strings containing a complete list of missing IAM scopes necessary to successfully execute the request.
     * Should be set if the API returns a 403 - Forbidden response in case of missing OAuth scopes.
     * */
    missingScopes?: string[];
    /**
     * Must be an array of strings containing a complete list of missing IAM permissions necessary to successfully execute the request.
     * Should be set if the API returns a 403 - Forbidden response in case of missing OAuth user permissions.
     * */
    missingPermissions?: string[];
}
/**
 * Contains information about an input parameter (path, query or request body)
 * that violated some validation rule of the service API and caused the warning.<br>
 * May contain additional fields further describing the warning.
 * */
export interface ConstraintViolation extends Record<string, string | undefined> {
    /**
     * Mandatory field message describing the warning.
     * */
    message: string;
    /**
     * Describes the general location of the violating parameter (query parameter, request body, etc.)
     */
    parameterLocation?: string;
    /**
     * Refers to the violating parameter within the parameterLocation.
     */
    path?: string;
}
